@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2005  The PARI group.
+Copyright (C) 2000-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -18,7 +18,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include <string.h>
 #include <ctype.h>
 #include "header.h"
-extern gpfunc listfunc[];
 
 gpdesc *newdesc(int nb)
 {
@@ -157,7 +156,7 @@ int readtypemode(FILE *f,char *buf, int len, int *mode)
 
 #define BUFFER_SIZE 1024
 
-void initdesc(char *descfile)
+void initdesc(const char *descfile)
 {
   char buf[BUFFER_SIZE];
   FILE *dfile;
@@ -205,13 +204,20 @@ void initdesc(char *descfile)
             {
               gpiterator *iter = &(da->iter);
               int niter=readnumber(dfile,buf,BUFFER_SIZE);
-              if (niter!=3) die(err_desc,"unsupported iterator");
+              if (niter<3 && niter>4) die(err_desc,"unsupported iterator");
               readentry(dfile,buf,BUFFER_SIZE);
               iter->type=strtotype(buf);
               readentry(dfile,buf,BUFFER_SIZE);
               iter->init=getfunc(buf); func = lfunc+nf;
               readentry(dfile,buf,BUFFER_SIZE);
               iter->next=getfunc(buf); func = lfunc+nf;
+              if (niter==4)
+              {
+                readentry(dfile,buf,BUFFER_SIZE);
+                iter->end=getfunc(buf); func = lfunc+nf;
+              }
+              else
+                iter->end=-1;
             }
             nargs=readnumber(dfile,buf,BUFFER_SIZE);
             da->nargs=nargs;
@@ -542,6 +548,16 @@ RPN(rpn_cast)
   pop(2);
 }
 
+RPN(rpn_prec)
+{
+  genprec(d->fout, preclevel);
+}
+
+RPN(rpn_bitprec)
+{
+  genbitprec(d->fout, preclevel);
+}
+
 RPN(rpn_code) { push(1); LVL(0)=-1; rpn_cast(d,stk); }
 
 RPN(rpn_wrapper)
@@ -575,7 +591,8 @@ RPN(rpn_cookie)
     gpfunc *gp = lfunc + findfunction(entryname(arg[0]));
     if (gp->spec==GPuser && gp->user->wrapper>=0)
     {
-      if (funcmode(*gp)&(1<<Mprec))
+      gpfunc *wr = lfunc + gp->user->wrapper;
+      if ((funcmode(*gp)&(1<<Mprec)) && !(funcmode(*wr)&(1<<Mprec)))
         genfuncbydesc(d->fout, nb-1,arg+1,FC_tovecprec,d->nerr);
       else
       {
@@ -651,6 +668,7 @@ const char *gencoderpn(FILE *fout, const char *p, gpdescarg *rule, int nbarg, in
     {"parens",0,rpn_parens},{"stdref",0,rpn_stdref},
     {"str_format",1,rpn_str_format},{"str_raw",1,rpn_str_raw},
     {"code",1,rpn_code},{"cast",2,rpn_cast},
+    {"prec",0,rpn_prec},{"bitprec",0,rpn_bitprec},
     {"format_string",1,rpn_format_string}, {"format_args",1,rpn_format_args},
     {"cookie",1,rpn_cookie}, {"wrapper",1,rpn_wrapper},
     {NULL,0,NULL}
