@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2005  The PARI group.
+Copyright (C) 2000-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -18,8 +18,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include <string.h>
 #include <ctype.h>
 #include "header.h"
-extern int indent;
-int currfunc;
 static int newcvar=1;
 
 /*Use getfunc or newuserfunc instead*/
@@ -126,12 +124,46 @@ int getfunc(const char *gpname)
   return newfunc(strdup(gpname));
 }
 
+void genprec(FILE *fout, int level)
+{
+  if (level)
+  {
+    char buf[64];
+    sprintf(buf, "bitprec%d", level);
+    if (getvarbyname(buf)>=0)
+      fprintf(fout, "nbits2prec(%s)",buf);
+    else if (getvarbyname(buf+3)>=0)
+      fprintf(fout, "%s",buf+3);
+    else die(-1,"Internal error: prec not found!");
+  }
+  else
+    fprintf(fout, "prec");
+}
+
+void genbitprec(FILE *fout, int level)
+{
+  if (level)
+  {
+    char buf[64];
+    sprintf(buf, "bitprec%d", level);
+    if (getvarbyname(buf)>=0)
+      fprintf(fout, "%s",buf);
+    else if (getvarbyname(buf+3)>=0)
+      fprintf(fout, "prec2nbits(%s)",buf+3);
+    else die(-1,"Internal error: prec not found!");
+  }
+  else
+    fprintf(fout, "prec2nbits(prec)");
+}
 void genautoarg(FILE *fout, char c, int nerr)
 {
   switch(c)
   {
   case 'p':
-    fprintf(fout,"prec");
+    genprec(fout, preclevel);
+    break;
+  case 'b':
+    genbitprec(fout, preclevel);
     break;
   case 'P':
     fprintf(fout,"precdl");
@@ -147,9 +179,14 @@ int genarg(int nerr, FILE *fout, char c, int n, PPproto pr)
     die(n,"missing mandatory argument in function call");
   switch(c)
   {
+  case 'I':
+  case 'E':
   case 'G':
     gencast(fout,n,Ggen);
     return 1;
+  case 'U':
+    gencast(fout,n,Gusmall);
+    return 1;
   case 'L':
   case 'P':
     gencast(fout,n,Gsmall);
@@ -162,6 +199,8 @@ int genarg(int nerr, FILE *fout, char c, int n, PPproto pr)
     gencode(fout,n);
     return 1;
   case 'W':
+    if (tree[n].f==Findarg)
+      n=tree[n].x;
     gencode(fout,n);
     return 1;
   case 'r':
@@ -186,6 +225,7 @@ void gendefarg(int n, FILE *fout, char c, const char *name)
   case '&':
   case 'r':
   case 's':
+  case 'E':
   case 'I':
   case 'V':
     fprintf(fout,"NULL");
@@ -296,16 +336,27 @@ void genuserfunc(FILE *fout, int n, int nf)
   gpfunc *gp=lfunc+nf;
   userfunc *ufunc=gp->user;
   context *fc=block+gp->user->bl;
-  nb=genlistargs(n,arg,0,ufunc->narg);
+  int narg = ufunc->narg;
+  if (narg && fc->c[ufunc->sarg+narg-1].flag&(1<<Cvararg))
+    nb=genlistargs(n,arg,0,STACKSZ);
+  else
+    nb=genlistargs(n,arg,0,narg);
   fprintf(fout,"%s(",gp->proto.cname);
-  for(j=0;j<ufunc->narg;j++)
+  for(j=0;j<narg;j++)
   {
     ctxvar *v=fc->c+ufunc->sarg+j;
     int t=vartype(*v);
     if (firstarg)
       fprintf(fout,", ");
     firstarg=1;
-    if (j<nb && arg[j]>=0 && arg[j]!=GNOARG)
+    if (v->flag&(1<<Cvararg))
+    {
+      int k;
+      for (k=j;k<nb;k++)
+        if (arg[k]==GNOARG) arg[k]=GNIL;
+      genfuncbydesc(fout,nb-j,arg+j,FC_tovec,nf);
+    }
+    else if (j<nb && arg[j]>=0 && arg[j]!=GNOARG)
       gencast(fout,arg[j],t);
     else if (v->flag&(1<<Cdefmarker))
       gencodenoarg(fout,t,n);
@@ -318,7 +369,7 @@ void genuserfunc(FILE *fout, int n, int nf)
   {
     if (firstarg) fprintf(fout,", ");
     else firstarg=1;
-    fprintf(fout,"prec");
+    genprec(fout, preclevel);
   }
   fprintf(fout,")");
 }
@@ -359,7 +410,12 @@ void genentryfunc(FILE *fout, int n)
     if (genfunc(fout,nb,stack,nf,n)==0)
       return;
     if (gp->dsc)
+    {
+      int i;
+      for(i=0;i<nb;i++)
+        fprintf(stderr,"%d: %s\n", i, GPname(tree[stack[i]].t));
       die(n," %s: arguments do not match descriptions",name);
+    }
     if (gp->proto.cname)
       name=gp->proto.cname;
   }
@@ -422,6 +478,8 @@ void gendecvar(FILE *fout, context *fc, int nerr)
 {
   int oldt=-1;
   int idx;
+  if (fc->s.n>0)
+    genlineno(fout, fc->c->node);
   for(idx=0;idx<fc->s.n;idx++)
   {
     ctxvar *v=fc->c+idx;
@@ -435,6 +493,8 @@ void gendecvar(FILE *fout, context *fc, int nerr)
     if (t!=oldt)
     {
       gendecvarend(fout,oldt);
+      if (idx<fc->s.n-1)
+        genlineno(fout, (v+1)->node);
       genindent(fout);
       if (genfuncbydesc1(fout,v->node,FC_decl_base,nerr))
         die(nerr,"type `%s' not suitable for a variable",GPname(t));
@@ -474,12 +534,16 @@ genprotoargs(FILE *fout, int nf, int firstarg, int nb, int *name)
     firstarg=1;
     switch(c)
     {
+    case 'b':
     case 'p':
     case 'P':
     case 'L':
     case 'n':
       fprintf(fout,"long");
       break;
+    case 'U':
+      fprintf(fout,"ulong");
+      break;
     case 'f':
       fprintf(fout,"long *");
       break;
@@ -496,20 +560,24 @@ genprotoargs(FILE *fout, int nf, int firstarg, int nb, int *name)
       break;
     case 'r':
     case 's':
-      fprintf(fout,"char *");
+      fprintf(fout,"const char *");
       break;
     default:
       die(err_func,"prototype letter `%c' not known",c);
     }
     if (name)
     {
-      if (i>=nb)
-        die(err_func,"too few arguments in lambda");
       fputs(" ",fout);
       if (c=='p')
         fprintf(fout,"prec");
+      else if (c=='b')
+        fprintf(fout,"bitprec");
       else
+      {
+        if (i>=nb)
+          die(err_func,"too few arguments in lambda");
         gencode(fout, name[i++]);
+      }
     }
   }
   if (name && i<nb)
@@ -568,7 +636,7 @@ void genprototype(FILE *fout, int nf, int kb)
 }
 
 static int
-genwrapargs(FILE *fout, int nf, int nb, int *name)
+genwrapargs(FILE *fout, int nf, int nb, int *name, long mode)
 {
   gpfunc *gp=lfunc+nf;
   int i = 0;
@@ -579,10 +647,9 @@ genwrapargs(FILE *fout, int nf, int nb, int *name)
   PPproto mod;
   while((mod=parseproto(&p,&c)))
   {
-    if (!firstarg) fprintf(fout,", ");
-    firstarg=0;
     switch(c)
     {
+    case 'b':
     case 'p':
     case 'P':
     case 'L':
@@ -604,12 +671,31 @@ genwrapargs(FILE *fout, int nf, int nb, int *name)
     default:
       die(err_func,"prototype letter `%c' not known",c);
     }
-    if (i>=nb)
-      die(err_func,"too few arguments in lambda");
-    if (c=='p')
-      fprintf(fout,"prec");
+    if (c=='p' || c=='b')
+    {
+      if (mode&(1<<Mprec))
+      {
+        if (!firstarg) fprintf(fout,", ");
+        firstarg=0;
+        if (c=='p')
+          genprec(fout, preclevel);
+        else
+          genbitprec(fout, preclevel);
+      }
+    }
     else
-      gencast(fout, name[i++], t);
+    {
+      int ot;
+      if (i>=nb)
+        die(err_func,"too few arguments in lambda");
+      if (!firstarg) fprintf(fout,", ");
+      firstarg=0;
+      ot = tree[name[i]].t;
+      tree[name[i]].t = t;
+      gencast(fout, name[i], ot);
+      tree[name[i]].t = ot;
+      i++;
+    }
   }
   if (name && i<nb)
     die(err_func,"too many arguments in lambda");
@@ -619,8 +705,9 @@ genwrapargs(FILE *fout, int nf, int nb, int *name)
 void genwrapper(FILE *fout, int nf, int wrap)
 {
   gpfunc *gp=lfunc+nf;
+  gpfunc *wr=lfunc+wrap;
   int t=functype(*gp), m=funcmode(*gp);
-  int has_prec=m&(1<<Mprec);
+  int has_prec=(m&(1<<Mprec)) && !(funcmode(*wr)&(1<<Mprec));
   int i,firstarg=1,d;
   int nerr=gp->user->defnode;
   int savc=s_ctx.n;
@@ -629,7 +716,6 @@ void genwrapper(FILE *fout, int nf, int wrap)
   int res;
   ctxvar *vres;
   context *fc=block+gp->user->bl;
-  gpfunc *wr=lfunc+wrap;
   gpdescarg *rule;
   if (!wr->proto.code)
     die(wr->node,"Wrapper not defined");
@@ -667,7 +753,7 @@ void genwrapper(FILE *fout, int nf, int wrap)
     fprintf(fout," = ");
   }
   fprintf(fout,"%s(",gp->proto.cname);
-  firstarg=genwrapargs(fout, wrap, nb, stack);
+  firstarg=genwrapargs(fout, wrap, nb, stack, m);
   for (i=0,d=1;i<fc->s.n;i++)
   {
     ctxvar *v=fc->c+i;
@@ -720,6 +806,7 @@ void gendeffunc(FILE *fout, int n)
     die(n,"Internal error in gendeffunc : func %s not found",name);
   gp=lfunc+currfunc;
   gencomment(fout,funcid,0);
+  genlineno(fout, funcid);
   genprototype(fout,currfunc,1);
   t=functype(*gp);
   if (t!=Ggen && t!=Gsmall)
@@ -739,6 +826,7 @@ void gendefblock(FILE *fout, int n)
   int i;
   int m;
   int savc;
+  int level = preclevel;
   context *fc=block+b;
   savc=s_ctx.n;
   for(i=0;i<fc->s.n;i++)
@@ -770,4 +858,5 @@ void gendefblock(FILE *fout, int n)
     genendbrace(fout,n);
   }
   s_ctx.n=savc;
+  preclevel = level;
 }
