@@ -22,25 +22,12 @@ Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#define GP2C_DECL
 #include "header.h"
 
-FILE *yyin;
-int linecount;
-int yydebug;
+extern int yydebug;
+extern FILE *yyin;
 static int optgen=0,opttree=0,opttype=0,optinfo=0;
-int optstrict=0;
-int optcleanvar=0;/*used in genbrace to suppress {} optimization*/
-int opthashline=0;
-int indentlevel=2;
-int preclevel=0;
-int do_warning=1;
-char *optprefix,*optsuffix;
-int debug;
-const char *nameparse, *namelib, *outfile;
-int lastpass;
-int autogc;
-int warn;
-int safecoeff;
 
 void dump(FILE *fout)
 {
@@ -72,11 +59,11 @@ void dump(FILE *fout)
       fprintf(fout,"\n");
     }
 }
-extern int indent;
 void
 init_compiler(void)
 {
   int n;
+  stack_init(&s_srcfile,sizeof(*srcfile),(void **)&srcfile);
   stack_init(&s_node,sizeof(*tree),(void **)&tree);
   stack_init(&s_value,sizeof(*value),(void **)&value);
   stack_init(&s_func,sizeof(*lfunc),(void **)&lfunc);
@@ -142,6 +129,7 @@ init_stdfunc(void)
     safecoeff=0;
   }
   FC_norange=findfuncdescopt("_norange");
+  FC_derivn=findfuncdescopt("_'_");
 }
 
 void
@@ -168,9 +156,11 @@ compile(FILE *fin, FILE *fout, const char *nom)
   int startnode;
   int status;
   int i;
+  linecount = 1;
+  filecount = 0;
+  nameparse = nom;
   init_compiler();
-  linecount=1;
-  nameparse=nom;
+  filecount = newsrcfile(strdup(nom));
   lib = genoutfile();
   tmplib=(char*)calloc(strlen(lib)+6,sizeof(*lib));
   sprintf(tmplib,"%s%s.so",lib[0]=='/'?"":"./",lib);
@@ -194,7 +184,7 @@ compile(FILE *fin, FILE *fout, const char *nom)
   for(i=0;i<s_errors.n;i++)
   {
     error_string *s=errors+i;
-    fprintf(stderr,"%s:%ld: %s\n",nameparse,s->lineno,s->txt);
+    fprintf(stderr,"%s:%ld: %s\n",srcfile[s->fileno],s->lineno,s->txt);
   }
   if(status)
   {
@@ -341,12 +331,19 @@ int main(int argc, char **argv)
   FILE *fin=stdin;
   FILE *fout=stdout;
   const char *infile = "stdin";
-  autogc=0;
-  safecoeff=0;
-  warn=0;
-  optprefix=NULL;
-  optsuffix=NULL;
+  autogc = 0;
+  safecoeff = 0;
+  optprefix = NULL;
+  optsuffix = NULL;
   outfile = NULL;
+  optcleanvar = 0;
+  opthashline = 0;
+  optstrict = 0;
+  indent = 0;
+  indentlevel = 2;
+  warn = 0;
+  do_warning = 1;
+  preclevel = 0;
   while((c=getopt(argc,argv,"gi:hflo:p:s:StvydCGLTVW"))!=-1)
   {
     switch(c)
