@@ -1,21 +1,19 @@
+/* A Bison parser, made by GNU Bison 3.0.4.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
+/* Bison interface for Yacc-like parsers in C
+
+   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
-   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -28,57 +26,66 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_YY_PARSE_H_INCLUDED
+# define YY_YY_PARSE_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     KDOTDOT = 258,
-     KPARROW = 259,
-     KARROW = 260,
-     KPE = 261,
-     KSE = 262,
-     KME = 263,
-     KDE = 264,
-     KDRE = 265,
-     KEUCE = 266,
-     KMODE = 267,
-     KAND = 268,
-     KOR = 269,
-     KID = 270,
-     KEQ = 271,
-     KNE = 272,
-     KGE = 273,
-     KLE = 274,
-     KSRE = 275,
-     KSLE = 276,
-     KSR = 277,
-     KSL = 278,
-     KDR = 279,
-     KPP = 280,
-     KSS = 281,
-     KINTEGER = 282,
-     KREAL = 283,
-     KENTRY = 284,
-     KSTRING = 285,
-     DEFFUNC = 286,
-     SEQ = 287,
-     LVAL = 288,
-     INT = 289,
-     SIGN = 290,
-     MAT = 291
-   };
+  enum yytokentype
+  {
+    KPARROW = 258,
+    KARROW = 259,
+    KDOTDOT = 260,
+    KPE = 261,
+    KSE = 262,
+    KME = 263,
+    KDE = 264,
+    KDRE = 265,
+    KEUCE = 266,
+    KMODE = 267,
+    KAND = 268,
+    KOR = 269,
+    KID = 270,
+    KEQ = 271,
+    KNE = 272,
+    KGE = 273,
+    KLE = 274,
+    KSRE = 275,
+    KSLE = 276,
+    KSR = 277,
+    KSL = 278,
+    KDR = 279,
+    KPP = 280,
+    KSS = 281,
+    KINTEGER = 282,
+    KREAL = 283,
+    KENTRY = 284,
+    KSTRING = 285,
+    SEQ = 286,
+    DEFFUNC = 287,
+    INT = 288,
+    LVAL = 289,
+    SIGN = 290,
+    DERIV = 291,
+    MAT = 292
+  };
 #endif
 /* Tokens.  */
-#define KDOTDOT 258
-#define KPARROW 259
-#define KARROW 260
+#define KPARROW 258
+#define KARROW 259
+#define KDOTDOT 260
 #define KPE 261
 #define KSE 262
 #define KME 263
@@ -104,36 +111,35 @@
 #define KREAL 283
 #define KENTRY 284
 #define KSTRING 285
-#define DEFFUNC 286
-#define SEQ 287
-#define LVAL 288
-#define INT 289
+#define SEQ 286
+#define DEFFUNC 287
+#define INT 288
+#define LVAL 289
 #define SIGN 290
-#define MAT 291
-
-
-
+#define DERIV 291
+#define MAT 292
 
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
-{
 
-/* Line 1676 of yacc.c  */
-#line 32 "parse.y"
+union YYSTYPE
+{
+#line 31 "parse.y" /* yacc.c:1909  */
 
   int val;
   strcom str;
 
+#line 133 "parse.h" /* yacc.c:1909  */
+};
 
-
-/* Line 1676 of yacc.c  */
-#line 131 "y.tab.h"
-} YYSTYPE;
+typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+
 extern YYSTYPE yylval;
 
+int yyparse (void);
 
+#endif /* !YY_YY_PARSE_H_INCLUDED  */
